#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/mount.h>
#include <linux/cdrom.h>
#include <linux/fs.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/MessageB.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/Text.h>
#include <Xm/Label.h>

/********************************************************************/

#define XCDC_VERSION         "1.0"

#define FG_COLOR             "black"
#define HI_COLOR             "red"

#define RB_FORM_HEIGHT       123
#define T_FORM_HEIGHT        110
#define B_FORM_HEIGHT         35
#define HEIGHT               231

#define T_WIDTH               30
#define WIDTH                190


String fallbacks[] =
{
  "Xcdc*CDROM_Device.value:             /dev/hdb",
  "Xcdc*Mount_Point.value:              /cdrom",
  "Xcdc*background:                     lightblue",
  "Xcdc*foreground:                     black",
  "Xcdc*selectColor:                    black",
  NULL
};

/********************************************************************/

void Callback_Change( Widget widget, XtPointer client_data, XtPointer call_data);
void Callback_Quit( Widget widget, XtPointer client_data, XtPointer call_data);
Widget MessageBox(Widget parent, int dialog_type, char *msg);
int mount_cd(char *devfile, char *mount_point);
int unmount_cd(char *devfile);

Widget  toplevel, topform, 
        rbframe, radiobox, togglebutton[10],
        iframe, irc, ilabel[10],
        tframe,trc,tbb,devlabel,devtext,mplabel,mptext,
        bframe, buttonrc, button[10];

char  msg[1000];
char  command[1000];
char *mount_point;
char *devfile;
int   which_cd;
int   rc;
int   prev_cd = -1;

/*********************************************************************/

int main(int argc, char *argv[])
 {
   Cardinal  n;
   ArgList   wargs;
   XtAppContext app;


   
   wargs = (ArgList)malloc(sizeof(Arg) * 30);
   /*---------------------------------------------------------*/
   /*    toplevel = XtInitialize(argv[0], "Xcdc", NULL, 0, &argc, argv); */
   toplevel = XtVaAppInitialize(&app, "Xcdc", NULL, 0, &argc, argv, fallbacks, NULL);

   if (argc > 1)
     if (!strcmp(argv[1], "-v"))
       {
	 printf("Xcdc CD Changer - Version " XCDC_VERSION "\n");
	 exit(0);
       }
     else
       {
	 printf("usage:  %s [-v]\n",argv[0]);
	 exit(0);
       }

   /*---------------------------------------------------------*/
   /* Topform --- top level that holds everything             */
   /*---------------------------------------------------------*/
   n = 0;
   XtSetArg(wargs[n], XmNwidth,  WIDTH);  n++;
   XtSetArg(wargs[n], XmNheight, HEIGHT); n++;
   topform = XmCreateForm(toplevel, "topform", wargs, n);

   /*---------------------------------------------------------*/
   /* Info frame                                              */
   /*---------------------------------------------------------*/
   n = 0;
   XtSetArg(wargs[n], XmNtopAttachment,XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNrightAttachment,  XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNwidth,  WIDTH/2); n++;
   XtSetArg(wargs[n], XmNheight, RB_FORM_HEIGHT); n++;
   iframe = XmCreateFrame(topform,"iframe",wargs,n);

   /*---------------------------------------------------------*/
   /* Info rowcolumn and labels                               */
   /*---------------------------------------------------------*/
   n=0;
   XtSetArg(wargs[n], XmNpacking,XmPACK_COLUMN);          n++;
   XtSetArg(wargs[n], XmNorientation,XmVERTICAL);         n++;
   XtSetArg(wargs[n], XmNnumColumns,1);                   n++;
   irc = XmCreateRowColumn(iframe,"iRC", wargs, n);

   ilabel[0] = XmCreateLabel(irc, "              ", NULL, 0); 
   ilabel[1] = XmCreateLabel(irc, "     Xcdc     ", NULL, 0); 
   ilabel[2] = XmCreateLabel(irc, "CD-ROM Changer", NULL, 0); 
   ilabel[3] = XmCreateLabel(irc, "  Version "XCDC_VERSION" ", NULL, 0); 
   ilabel[4] = XmCreateLabel(irc, "              ", NULL, 0); 

   /*---------------------------------------------------------*/
   /* Radio button frame                                      */
   /*---------------------------------------------------------*/
   n = 0;
   XtSetArg(wargs[n], XmNtopAttachment,XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNleftAttachment,  XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNwidth,  WIDTH/2); n++;
   XtSetArg(wargs[n], XmNheight, RB_FORM_HEIGHT); n++;
   rbframe = XmCreateFrame(topform,"rbframe",wargs,n);

   /*---------------------------------------------------------*/
   /* Radio box and buttons                                   */
   /*---------------------------------------------------------*/
   radiobox   = XmCreateRadioBox(rbframe, "radiobox", NULL, 0);
   n=0;
   togglebutton[0] = XmCreateToggleButton( radiobox," CD 0 ",wargs, n);
   togglebutton[1] = XmCreateToggleButton( radiobox," CD 1 ",wargs, n);
   togglebutton[2] = XmCreateToggleButton( radiobox," CD 2 ",wargs, n);
   togglebutton[3] = XmCreateToggleButton( radiobox," CD 3 ",wargs, n);

   /* XmToggleButtonSetState(togglebutton[0],TRUE, FALSE);  */
   XmToggleButtonSetState(togglebutton[0],FALSE,FALSE);
   XmToggleButtonSetState(togglebutton[1],FALSE,FALSE);
   XmToggleButtonSetState(togglebutton[2],FALSE,FALSE);
   XmToggleButtonSetState(togglebutton[3],FALSE,FALSE);
   
   /*---------------------------------------------------------*/
   /* Button Frame and Buttons                                */
   /*---------------------------------------------------------*/
   n = 0;
   XtSetArg(wargs[n], XmNleftAttachment,  XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNbottomAttachment,XmATTACH_FORM); n++; 
   XtSetArg(wargs[n], XmNwidth,  WIDTH);                  n++;
   XtSetArg(wargs[n], XmNheight, B_FORM_HEIGHT);          n++;
   bframe = XmCreateFrame(topform,"bframe",wargs,n);

   buttonrc  = XmCreateRowColumn(bframe,"ButtonFrame", NULL, 0);
   button[0] = XmCreatePushButton(buttonrc,"   Change    ",NULL,0);
   button[1] = XmCreatePushButton(buttonrc,"    Quit     ",NULL,0);
   XtAddCallback( button[0], XmNactivateCallback, Callback_Change, (XtPointer) 0);	  
   XtAddCallback( button[1], XmNactivateCallback, Callback_Quit,   (XtPointer) 0);	  

   /*---------------------------------------------------------*/
   /* Text rc and text box                                    */
   /*---------------------------------------------------------*/
   n = 0;
   XtSetArg(wargs[n], XmNleftAttachment,  XmATTACH_FORM); n++;
   XtSetArg(wargs[n], XmNbottomAttachment,XmATTACH_FORM); n++; 
   XtSetArg(wargs[n], XmNwidth,  WIDTH);                  n++;
   XtSetArg(wargs[n], XmNheight, T_FORM_HEIGHT);        n++;
   tframe = XmCreateFrame(topform,"tframe",wargs,n);

   n=0;
   XtSetArg(wargs[n], XmNpacking,XmPACK_COLUMN);          n++;
   XtSetArg(wargs[n], XmNorientation,XmVERTICAL);         n++;
   XtSetArg(wargs[n], XmNnumColumns,2);                   n++;
   trc = XmCreateRowColumn(tframe,"TextRC", wargs, n);

   devlabel = XmCreateLabel(trc, "CD Device:", NULL, 0); 
   mplabel  = XmCreateLabel(trc, "Mount Point:", NULL, 0); 

   n=0;
   XtSetArg(wargs[n], XmNwidth,  T_WIDTH);                n++;
   devtext = XmCreateText(trc, "CDROM_Device",  wargs, n); 

   n=0;
   XtSetArg(wargs[n], XmNwidth,  T_WIDTH);                n++;
   mptext = XmCreateText(trc, "Mount_Point",  wargs, n); 


   /*---------------------------------------------------------*/
   /* Manage everything...                                    */
   /*---------------------------------------------------------*/
   XtManageChildren(togglebutton, 4);
   XtManageChild(radiobox);
   XtManageChild(rbframe);

   XtManageChildren(button, 2);
   XtManageChild(buttonrc);
   XtManageChild(bframe);

   XtManageChildren(ilabel, 5);
   XtManageChild(irc);
   XtManageChild(iframe);

   XtManageChild(devlabel);
   XtManageChild(mplabel);
   XtManageChild(devtext);
   XtManageChild(mptext);
   XtManageChild(trc);
   XtManageChild(tframe);

   XtManageChild(topform);
   XtRealizeWidget(toplevel);
   XtAppMainLoop(app);
   
   return(0);
 }
     
/*********************************************************************/

void Callback_Change(Widget widget, XtPointer client_data, XtPointer call_data) 
{
  int cdrom;

  if (XmToggleButtonGetState(togglebutton[0]) == True)
    which_cd = 0;
  else if (XmToggleButtonGetState(togglebutton[1]) == True)
    which_cd = 1;
  else if (XmToggleButtonGetState(togglebutton[2]) == True)
    which_cd = 2;
  else if (XmToggleButtonGetState(togglebutton[3]) == True)
    which_cd = 3;
  else
    {
      MessageBox(toplevel,XmDIALOG_ERROR,"You must first select a CD to change to!");
      return;
    }

  devfile = XmTextGetString(devtext);
  if (!devfile || devfile[0]=='\0')
    {
      MessageBox(toplevel,XmDIALOG_ERROR,"You must specify a CD device (eg. /dev/hdb)!");
      return;
    }

  mount_point = XmTextGetString(mptext);
  if (!mount_point)
    mount_point = "";

  
  sprintf(command,"mount | grep -q %s",devfile);
  if (system(command) == 0)      /* if already mounted, unmount it first */
    {
      rc = unmount_cd(devfile);
      if (rc != 0)
	{
	  sprintf(msg,"%s was mounted; could not unmount!",devfile);
	  MessageBox(toplevel,XmDIALOG_ERROR,msg);
	  return;
	}
    }
      
  close(cdrom);              /* just as a precaution... */
  
  if (!(cdrom = open(devfile,0)))
    {
      sprintf(msg,"Could not open the requested CD device! error=[%s]",strerror(errno));
      MessageBox(toplevel,XmDIALOG_ERROR,msg);
      return;
    }

  if (ioctl(cdrom,CDROMLOADFROMSLOT,which_cd)) 
    {
      sprintf(msg,"Could not load the requested CD! error=[%s]",strerror(errno));
      MessageBox(toplevel,XmDIALOG_ERROR,msg);
      close(cdrom);
      return;
    }
  
  if (close(cdrom))
    {
      sprintf(msg,"Could not close the requested CD device! error=[%s]",strerror(errno));
      MessageBox(toplevel,XmDIALOG_ERROR,msg);
      return;
    }

  if (mount_point && mount_point[0])
    mount_cd(devfile, mount_point);

  /* Un-hilite the previously selected slot and hilite the new one */
  if (prev_cd >= 0)
    {
      XtVaSetValues(togglebutton[prev_cd], XtVaTypedArg, XmNforeground, XmRString, 
		    FG_COLOR, strlen(FG_COLOR)+1,NULL);
      XtVaSetValues(togglebutton[prev_cd], XtVaTypedArg, XmNselectColor, XmRString, 
		    FG_COLOR, strlen(FG_COLOR)+1,NULL);
    }
  XtVaSetValues(togglebutton[which_cd], XtVaTypedArg, XmNforeground, XmRString, 
		HI_COLOR, strlen(HI_COLOR)+1,NULL);
  XtVaSetValues(togglebutton[which_cd], XtVaTypedArg, XmNselectColor, XmRString, 
		HI_COLOR, strlen(HI_COLOR)+1,NULL);
  

  prev_cd = which_cd;
}

/*********************************************************************/

void Callback_Quit( Widget widget, XtPointer client_data, XtPointer call_data) 
{
  exit(0);
}

/*********************************************************************/

int mount_cd(char *devfile, char *mount_point)
{
  sprintf(command,"mount %s",devfile);
  rc = system(command);
  if (rc)
    {
      sprintf(msg,"RC=%d: Failure issuing '%s'", rc, command);
      MessageBox(toplevel,XmDIALOG_ERROR,msg);
      return(errno);
    }
  return(0);
}

/*********************************************************************/

int unmount_cd(char *devfile)
{
  sprintf(command,"umount %s",devfile);
  rc = system(command);
  return(rc);
}

/*********************************************************************/

Widget MessageBox(Widget parent, int dialog_type, char *msg)
 {
  Widget dialog;
  XmString text;

  
  dialog = XmCreateMessageDialog(parent,"PopUp",NULL,0);
  text   = XmStringCreateSimple(msg);
  XtVaSetValues(dialog,
		XmNdialogType,    dialog_type,
		XmNmessageString, text,
		NULL);
  XmStringFree(text);
  XtManageChild(dialog);
  XtPopup(XtParent(dialog),XtGrabNone);
  return(dialog);
 }

/*********************************************************************/

